2D Game Engines for the Web: An Overview
-------------------------------------
You've probably seen this happen before: new developer wants to write a game. This developer looks around to see what tools are out there, and is dissapointed by the options. So this developer decides to create a custom engine that will fill all of the needs for the current game idea, and all future ones anyone will ever have ever.

And this developer never makes an actual game. 

It's a tragedy that happens far too often. But luckily for us, that story has spawned dozens of amazing game engines for all possible platforms, with a wide range of methologies, tools, and licensing models. For example, at the time of this writing [Javascripting.com](javascripting.com) list fourteen dedicated game libraries, not including physics engines, audio libraries, and specialized input systems for games. And those are just the Open Source ones. Take it from us - don't write your own tools. Write your own games instead!

In this article we'll walk through three popular Open Source 2D game engines for the web: Crafty, Pixi, and Phaser. To compare and contrast these libraries, we're going to port our [Ping] game from our very first article over to each of the three and see how it feels. Keep in mind by restricting ourselves to 2D, we're leaving out a bunch of cool game engines for the web. No worries, we'll be covering those shortly. For now, let's focus on two dimensions and see what great stuff is out there.


Crafty.js
---------
Crafty is meant primarily for tile-based games, although it works very well for a wide variety of 2D game types, including our Ping game, simple as it is. Crafty's foundation is atop a nice object model/dependency system that it calls *components*. Components are like classes you define, with specific attributes & actions, and each instance of a component is called an *entity*. The interesting part, though, is that components are meant to be mixed together heavily to make complex, feature-rich entities. For example, if you want to make a sprite sheet in Crafty, you use the `Crafty.sprite()` function to generate a bunch of components that represent your sprites. Here is where we define the sprite types from our image:
```
Crafty.sprite("sprites.png", 
	{PlayerSprite: [0, 128, 128, 128],
	 OpponentSprite: [128, 128, 128, 128],
	 BallSprite: [0, 64, 64, 64],
	 UpSprite: [64,64,64,64],
	 DownSprite: [128,64,64,64],
	 LeftSprite: [192,64,64,64],
	 RightSprite: [256,64,64,64]});
```
Now, if you want to make a ball entity that uses the image of the ball on the sprite sheet, you include the `"BallSprite"` component in the creation of the entity, using the `Crafty.e()` function:
```
Crafty.e("Ball, BallSprite");
```
Now, this won't draw it to the screen just yet - you have to tell Crafty that you want this entity to live in 2D (e.g. have an `x` and `y` position), and that you want it to draw using DOM elements:
```
Crafty.e("Ball, 2D, DOM, BallSprite");
```
If you wanted to draw your elements in a canvas, it's as simple as replacing the `DOM` component with `Canvas`. 

Entities then act very much like independent objects on your screen by reacting to events. Like standard JavaScript, Crafty entities have a `bind()` function for reacting to events, although it's important to know that these are Crafty-specific events. For example, if you want an entity to do something every frame, have it react to the `EnterFrame` event. This is exactly what's needed for the ball, to move it around based on who is holding it, and physics if necessary:
```
Crafty.e("Ball, 2D, DOM, BallSprite, Collision")
	.attr({ x: width/2, y: height/2, velocity: [0,0], owner: 'User' })
	.bind('EnterFrame', function () {
			// If there's an owner, have the ball follow the owner.
			// We nudge the ball to the left or right of the player
			// so it doesn't overlap
			if (this.owner) {
				var owner = Crafty(this.owner);
				switch(this.owner) {
						case 'User':
							this.x = owner.x + 128;
							break;
						case 'Opponent':
							this.x = owner.x - 64;
				}
				this.y = owner.y + 32;
				return;
			}

			// Bounce the ball off the ceiling or floor
			if (this.y <= 0 || this.y >= (height - 64))
					this.velocity[1] = -this.velocity[1];

			// Move the ball based on its velocity, which is defined in 
			// pixels per millisecond.
			var millis = 1000 / Crafty.timer.FPS();
			this.x += this.velocity[0] * millis;
			this.y += this.velocity[1] * millis;
	})
```
If you look closely, you'll see a lot of usage of `this` to refer to entity properties. In this case, there are the built-in Crafty `this.x` and `this.y`, which define the position of the ball. This functionality is added when we create the ball with the `"2D"` component. `this.velocity` is custom  - you can see it defined as a property using the `attr()` function. Same with `this.owner`, which is used to figure out if either of the players is holding the ball.

We don't use them a huge amount, but Crafty has a lot of built-in components for all kinds of great functionality. Here are just a few:

* __Gravity__: Crafty has a simple gravity engine that will automatically pull an entity downward, and you can define any number of element types that will stop its motion. 
* __TwoWay / FourWay / MultiWay Motion__: With these components, you get a variety of arcade-style input methods for motion. *TwoWay* is for platformers, with left, right, and jump options bindable to whatever keys you want. *FourWay* allows for top-down motion in the cardinal directions. *Multiway* allows custom definition of a number of inputs, each with an arbitrary direction.
* __Particles__: Crafty includes a fast particle system, usable only with canvas drawing. As soft, round, single-color images, the particles are particularly suited for smoke/fire/explosions. They are quite fun to tweak to make some neat effects.

We've posted the code of Ping, implemented in Crafty.js [here](link). We recommend you take a look to get a full picture of how we ported things, particularly the AI for the opponent. And of course you can check out [Crafty.js at their website](craftyjs.com).

Pixi.js
------------
Pixi is the barebones, close-to-the-metal 2D renderer for the web. Pixi can forego using the 2D canvas and dives straight into WebGL, giving it blazingly fast performance. A Pixi game has two key pieces: a stage, which describes the layout of your game in a scene graph, and a renderer, which actually draws the elements in the stage to the screen using canvas or WebGL. By default, Pixi will use WebGL if it's available.
```
$(document).ready(function() {
	// The stage to play your game
 	stage = new PIXI.Stage(0xFFFFFFFF);
	lastUpdate = 0;

  // autoDetectRenderer() finds the fastest renderer you've got.
  renderer = PIXI.autoDetectRenderer(innerWidth, innerHeight);
  document.body.appendChild(renderer.view);
	
	//...
});
```
Pixi has a scenegraph and fast transformations, but it's important to note that you need to render the scene yourself. So for our implementation of Ping, we keep the rendering loop using `requestAnimationFrame()` the same:
```
function update(time) {
  var t = time - lastUpdate;
  lastUpdate = time;

  ball.update(t);
  ai.update();
	
  // New! We actually have to render the scene
  renderer.render(stage);

  requestAnimationFrame(update);
}
```
Pixi doesn't dictate much about how to structure your game like Crafty does, so we can pretty much use the same code from our Ping game, with some slight modifications.

The first big difference we see is that Pixi either lets you load sprites as images from files one-by-one, or using a tiled image using a special tool called [TexturePacker](https://www.codeandweb.com/texturepacker). This tool will automatically merge a bunch of individual images into a single, optimized file with an accompanying JSON file that describes the locations of the images. This is a little clunky for a small project like ours, but TexturePacker has a lot of great features for making 2D games and integrates well into a number of frameworks. We've used the tool to make our spritesheet, so let's use it! Here's our updated `ready()` function, loading the sprite images, a.k.a. the `frames`:
```
$(document).ready(function() {
	// create an new instance of a pixi stage
 	stage = new PIXI.Stage(0xFFFFFFFF);
	lastUpdate = 0;

  // create a renderer instance.
  renderer = PIXI.autoDetectRenderer(innerWidth, innerHeight);
  document.body.appendChild(renderer.view);
	
	var images = ["sprites.json"];
  var loader = new PIXI.AssetLoader(images);
	loader.onComplete = startGame;
  loader.load(); 
});
```
Note the loader is asynchronous, as it requires us to provide a function to call when it's done loading. As such the remaining initialization for our game, like creating our player, opponent, and ball, is in a new `startGame()` function. To use these newly initialized frames, we can use the `PIXI.Sprite.fromFrame()` method, which takes an index into the array of sprites in the JSON file.

One last difference to note is that Pixi provides a very nice touch and mouse input system. To make any sprite input-ready, you simply set the `interactive` property to `true` and add some event handlers directly to the sprite. For instance, for our up/down buttons, we bind events to move the player up and down like so:

```
up = new PIXI.Sprite.fromFrame(3);
up.interactive = true;
stage.addChild(up);
up.touchstart = up.mousedown = function() {
	player.move(-distance);
}

down = new PIXI.Sprite.fromFrame(4);
down.interactive = true;
stage.addChild(down);
down.touchstart = down.mousedown = function() {
	player.move(distance);
}
```
It's important to note that we have to actually add the sprite to the stage to make it draw! 

In summary, check out Pixi.js at [pixijs.com](pixijs.com). Pixi.js is fast, minimal, and easy to use. Also check out our version of Ping built in Pixi [here](). Our only issue with Pixi right now is that it doesn't give you a lot when it comes to making games specifically. For that you'll need a little something more...

Phaser.io
---------
And that little bit more is Phaser. Phaser is a fully-featured game engine that uses Pixi under the hood to do all its rendering. Phaser adds a bunch of great features - simplified animation using frames, a game state system, music & audio, and physics with not one but three different contributing libraries. 

Considering that we already have a version of our game that works in Pixi, porting to Phaser should be a snap. Phaser makes object creation even easier than in Pixi:
```
ball = game.add.sprite(x, y, 'sprites');
```
While Phaser can be a bit more concise, there are places where things are a little more complex. For instance, that last parameter, `'sprite'`, refers to an image loaded at startup: 
```
game.load.image('sprites', 'sprites.png');
```
Phaser has a very cool sprite sheet system for images, where you can break up an image into tiles. To do this, you would call something like this:
```
game.load.spritesheet('sprites', 'mySprites.png', spriteWidth, spriteHeight, totalSprites);
```
So why do we just reference one file? The second line of code above assumes all sprites are the same size, and our sprite sheet has sprites of 64 pixels and 128 pixels. So we have to do the cropping of the image manually. To crop down our image for the ball, we set a cropping rectangle on the sprite:
```
ball.cropRect = new Phaser.Rectangle(0, 64, 64, 64);
ball.updateCrop();
```
Other than that, Phaser is quite powerful and flexible. You can make your game be event-based, or you can check for events and respond to them sequentially in an `update()` function. For instance, to handle a key input, you can either respond to an event:
```
game.input.keyboard.onDownCallback = function(key) {
	console.log(key + " was pressed.");
}
```
or check if the key is pressed in your update loop:
```
function update() {
	if (game.input.keyboard.isDown('a'.charCodeAt(0)) {
		console.log("'A' key was pressed");
	}
}
```
This goes for a lot of events in Phaser, enabling sequential or asynchronous coding depending on your preference or your need. Just so you know, though, on occasion Phaser provides only the latter form of event handling. A good example is the *Arcade* physics system, where you must explicitly make a function call every update to check if collisions between objects have occurred:
```
game.physics.arcade.collide(player, ball, function() {
	ballOwner = player;
});
```
This code determines if the ball has come in contact with our player sprite, and gives control of the ball over to the player when that happens. Check out [Phaser.io on their website](phaser.io), and check out our implementation of Ping on Phaser [here]().

Conclusion
----------
If you're looking for dedicated 2D game engines for JavaScript, there are a lot of options out there to fit your needs. Here are some things to keep in mind when picking a framework:

* Don't be dazzled by lots of features, all-encompassing monolithic frameworks, and other bells and whistles. Unless the bells and whistles address specific aspects of the game you want to make, they'll probably get in the way. Also consider how the features are componentized - will the whole system still work if you remove the physics system? Or does the framework rely on the physics system to provide other functionality?
* Try to understand how you can customize or extend the framework. There's a good likelihood that if you're making something slightly out of the ordinary, you'll need to write custom code that relies heavily on the framework you choose. At that point you can either write a whole new, independent feature, or extend the functionality of the framework. If you extend the framework itself rather than adding on top of it, you'll end up with more maintainable code.

We think the frameworks above cover many of these issues, so definitely check them out. Stay tuned as we'll start diving into 3D engines, which have a lot more exciting things to show off!